/**
 * @licstart The following is the entire license notice for the
 * Javascript code in this page
 *
 * Copyright 2018 Mozilla Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @licend The above is the entire license notice for the
 * Javascript code in this page
 */
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PdfFileLoader = exports.PDFPrintServiceFactory = exports.DefaultExternalServices = exports.PDFViewerApplication = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ui_utils = require('./ui_utils');

var _pdf = require('../pdf');

var _pdf_cursor_tools = require('./pdf_cursor_tools');

var _pdf_rendering_queue = require('./pdf_rendering_queue');

var _pdf_sidebar = require('./pdf_sidebar');

var _app_options = require('./app_options');

var _dom_events = require('./dom_events');

var _overlay_manager = require('./overlay_manager');

var _password_prompt = require('./password_prompt');

var _pdf_file_loader = require('./pdf_file_loader');

var _pdf_find_bar = require('./pdf_find_bar');

var _pdf_find_controller = require('./pdf_find_controller');

var _pdf_history = require('./pdf_history');

var _pdf_link_service = require('./pdf_link_service');

var _pdf_outline_viewer = require('./pdf_outline_viewer');

var _pdf_sidebar_resizer = require('./pdf_sidebar_resizer');

var _pdf_thumbnail_viewer = require('./pdf_thumbnail_viewer');

var _pdf_viewer = require('./pdf_viewer');

var _secondary_toolbar = require('./secondary_toolbar');

var _toolbar = require('./toolbar');

var _view_history = require('./view_history');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_SCALE_DELTA = 1.1;
var DISABLE_AUTO_FETCH_LOADING_BAR_TIMEOUT = 5000;
var FORCE_PAGES_LOADED_TIMEOUT = 10000;
function loadFakeWorker() {
  if (!_pdf.GlobalWorkerOptions.workerSrc) {
    _pdf.GlobalWorkerOptions.workerSrc = _app_options.AppOptions.get('workerSrc');
  }
  return (0, _pdf.loadScript)(_pdf.PDFWorker.getWorkerSrc());
}
var DefaultExternalServices = {
  updateFindControlState: function updateFindControlState(data) {},
  updateFindMatchesCount: function updateFindMatchesCount(data) {},
  initPassiveLoading: function initPassiveLoading(callbacks) {},
  fallback: function fallback(data, callback) {},
  reportTelemetry: function reportTelemetry(data) {},
  createDownloadManager: function createDownloadManager(options) {
    throw new Error('Not implemented: createDownloadManager');
  },
  createPreferences: function createPreferences() {
    throw new Error('Not implemented: createPreferences');
  },
  createL10n: function createL10n(options) {
    throw new Error('Not implemented: createL10n');
  },

  supportsIntegratedFind: false,
  supportsDocumentFonts: true,
  supportsDocumentColors: true,
  supportedMouseWheelZoomModifierKeys: {
    ctrlKey: true,
    metaKey: true
  }
};

var PDFViewerApplication = function () {
  function PDFViewerApplication() {
    _classCallCheck(this, PDFViewerApplication);

    this.fellback = false;
    this.initialBookmark = null;
    this.initialized = false;
    this.appConfig = null;
    this.pdfDocument = null;
    this.pdfLoadingTask = null;
    this.printService = null;
    this.pdfViewer = null;
    this.pdfThumbnailViewer = null;
    this.pdfRenderingQueue = null;
    this.pdfPresentationMode = null;
    this.pdfDocumentProperties = null;
    this.pdfLinkService = null;
    this.pdfHistory = null;
    this.pdfSidebar = null;
    this.pdfSidebarResizer = null;
    this.pdfOutlineViewer = null;
    this.pdfAttachmentViewer = null;
    this.pdfCursorTools = null;
    this.store = null;
    this.downloadManager = null;
    this.preferences = null;
    this.toolbar = null;
    this.secondaryToolbar = null;
    this.eventBus = null;
    this.l10n = null;
    this.isInitialViewSet = false;
    this.downloadComplete = false;
    this.isViewerEmbedded = window.parent !== window;
    this.url = '';
    this.baseUrl = '';
    this._boundEvents = {};
    this.contentDispositionFilename = null;
    this.pdfFileLoader = null;
    this.overlayManager = new _overlay_manager.OverlayManager();
    this.eventHandler = new PDFViewerApplicationEventHandler(this);
  }

  _createClass(PDFViewerApplication, [{
    key: 'initialize',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee(appConfig) {
        var _this = this;

        var appContainer;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                this.preferences = PDFViewerApplication.externalServices.createPreferences();
                this.appConfig = appConfig;
                _context.next = 4;
                return this._readPreferences();

              case 4:
                _context.next = 6;
                return this._parseHashParameters();

              case 6:
                _context.next = 8;
                return this._initializeL10n();

              case 8:
                if (this.isViewerEmbedded && _app_options.AppOptions.get('externalLinkTarget') === _pdf.LinkTarget.NONE) {
                  _app_options.AppOptions.set('externalLinkTarget', _pdf.LinkTarget.TOP);
                }
                _context.next = 11;
                return this._initializeViewerComponents();

              case 11:
                this.bindEvents();
                this.bindWindowEvents();
                appContainer = appConfig.appContainer || document.documentElement;

                this.l10n.translate(appContainer).then(function () {
                  _this.eventBus.dispatch('localized', { source: _this });
                });
                this.initialized = true;

              case 16:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function initialize(_x) {
        return _ref.apply(this, arguments);
      }

      return initialize;
    }()
  }, {
    key: '_readPreferences',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        var OVERRIDES, prefs, name;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                OVERRIDES = {
                  disableFontFace: true,
                  disableRange: true,
                  disableStream: true,
                  textLayerMode: _ui_utils.TextLayerMode.DISABLE
                };
                _context2.prev = 1;
                _context2.next = 4;
                return this.preferences.getAll();

              case 4:
                prefs = _context2.sent;
                _context2.t0 = _regenerator2.default.keys(prefs);

              case 6:
                if ((_context2.t1 = _context2.t0()).done) {
                  _context2.next = 13;
                  break;
                }

                name = _context2.t1.value;

                if (!(name in OVERRIDES && _app_options.AppOptions.get(name) === OVERRIDES[name])) {
                  _context2.next = 10;
                  break;
                }

                return _context2.abrupt('continue', 6);

              case 10:
                _app_options.AppOptions.set(name, prefs[name]);
                _context2.next = 6;
                break;

              case 13:
                _context2.next = 17;
                break;

              case 15:
                _context2.prev = 15;
                _context2.t2 = _context2['catch'](1);

              case 17:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[1, 15]]);
      }));

      function _readPreferences() {
        return _ref2.apply(this, arguments);
      }

      return _readPreferences;
    }()
  }, {
    key: '_parseHashParameters',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        var waitOn, hash, hashParams, viewer, enabled;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (_app_options.AppOptions.get('pdfBugEnabled')) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return');

              case 2:
                waitOn = [];
                hash = document.location.hash.substring(1);
                hashParams = (0, _ui_utils.parseQueryString)(hash);

                if ('disableworker' in hashParams && hashParams['disableworker'] === 'true') {
                  waitOn.push(loadFakeWorker());
                }
                if ('disablerange' in hashParams) {
                  _app_options.AppOptions.set('disableRange', hashParams['disablerange'] === 'true');
                }
                if ('disablestream' in hashParams) {
                  _app_options.AppOptions.set('disableStream', hashParams['disablestream'] === 'true');
                }
                if ('disableautofetch' in hashParams) {
                  _app_options.AppOptions.set('disableAutoFetch', hashParams['disableautofetch'] === 'true');
                }
                if ('disablefontface' in hashParams) {
                  _app_options.AppOptions.set('disableFontFace', hashParams['disablefontface'] === 'true');
                }
                if ('disablehistory' in hashParams) {
                  _app_options.AppOptions.set('disableHistory', hashParams['disablehistory'] === 'true');
                }
                if ('webgl' in hashParams) {
                  _app_options.AppOptions.set('enableWebGL', hashParams['webgl'] === 'true');
                }
                if ('useonlycsszoom' in hashParams) {
                  _app_options.AppOptions.set('useOnlyCssZoom', hashParams['useonlycsszoom'] === 'true');
                }
                if ('verbosity' in hashParams) {
                  _app_options.AppOptions.set('verbosity', hashParams['verbosity'] | 0);
                }

                if (!('textlayer' in hashParams)) {
                  _context3.next = 23;
                  break;
                }

                _context3.t0 = hashParams['textlayer'];
                _context3.next = _context3.t0 === 'off' ? 18 : _context3.t0 === 'visible' ? 20 : _context3.t0 === 'shadow' ? 20 : _context3.t0 === 'hover' ? 20 : 23;
                break;

              case 18:
                _app_options.AppOptions.set('textLayerMode', _ui_utils.TextLayerMode.DISABLE);
                return _context3.abrupt('break', 23);

              case 20:
                viewer = this.appConfig.viewerContainer;

                viewer.classList.add('textLayer-' + hashParams['textlayer']);
                return _context3.abrupt('break', 23);

              case 23:
                if ('pdfbug' in hashParams) {
                  _app_options.AppOptions.set('pdfBug', true);
                  enabled = hashParams['pdfbug'].split(',');

                  waitOn.push(this.eventHandler.loadAndEnablePDFBug(enabled));
                }
                if ('locale' in hashParams) {
                  _app_options.AppOptions.set('locale', hashParams['locale']);
                }
                return _context3.abrupt('return', Promise.all(waitOn).catch(function (reason) {
                  console.error('_parseHashParameters: "' + reason.message + '".');
                }));

              case 26:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function _parseHashParameters() {
        return _ref3.apply(this, arguments);
      }

      return _parseHashParameters;
    }()
  }, {
    key: '_initializeL10n',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
        var dir;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                this.l10n = PDFViewerApplication.externalServices.createL10n({ locale: _app_options.AppOptions.get('locale') });
                _context4.next = 3;
                return this.l10n.getDirection();

              case 3:
                dir = _context4.sent;

                document.getElementsByTagName('html')[0].dir = dir;

              case 5:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function _initializeL10n() {
        return _ref4.apply(this, arguments);
      }

      return _initializeL10n;
    }()
  }, {
    key: '_initializeViewerComponents',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
        var appConfig, dispatchToDOM, eventBus, pdfRenderingQueue, pdfLinkService, findController, container, viewer, thumbnailContainer, sidebarConfig;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                appConfig = this.appConfig;
                dispatchToDOM = _app_options.AppOptions.get('eventBusDispatchToDOM');
                eventBus = appConfig.eventBus || (0, _dom_events.getGlobalEventBus)(dispatchToDOM);

                this.eventBus = eventBus;
                pdfRenderingQueue = new _pdf_rendering_queue.PDFRenderingQueue();

                pdfRenderingQueue.onIdle = this.cleanup.bind(this);
                this.pdfRenderingQueue = pdfRenderingQueue;
                pdfLinkService = new _pdf_link_service.PDFLinkService({
                  eventBus: eventBus,
                  externalLinkTarget: _app_options.AppOptions.get('externalLinkTarget'),
                  externalLinkRel: _app_options.AppOptions.get('externalLinkRel')
                });

                this.pdfLinkService = pdfLinkService;
                findController = new _pdf_find_controller.PDFFindController({
                  linkService: pdfLinkService,
                  eventBus: eventBus
                });

                this.findController = findController;
                container = appConfig.mainContainer;
                viewer = appConfig.viewerContainer;

                this.pdfViewer = new _pdf_viewer.PDFViewer({
                  container: container,
                  viewer: viewer,
                  eventBus: eventBus,
                  renderingQueue: pdfRenderingQueue,
                  linkService: pdfLinkService,
                  findController: findController,
                  renderer: _app_options.AppOptions.get('renderer'),
                  enableWebGL: _app_options.AppOptions.get('enableWebGL'),
                  l10n: this.l10n,
                  textLayerMode: _app_options.AppOptions.get('textLayerMode'),
                  imageResourcesPath: _app_options.AppOptions.get('imageResourcesPath'),
                  renderInteractiveForms: _app_options.AppOptions.get('renderInteractiveForms'),
                  enablePrintAutoRotate: _app_options.AppOptions.get('enablePrintAutoRotate'),
                  useOnlyCssZoom: _app_options.AppOptions.get('useOnlyCssZoom'),
                  maxCanvasPixels: _app_options.AppOptions.get('maxCanvasPixels')
                });
                pdfRenderingQueue.setViewer(this.pdfViewer);
                pdfLinkService.setViewer(this.pdfViewer);
                if (appConfig.sidebar) {
                  thumbnailContainer = appConfig.sidebar.thumbnailView;

                  this.pdfThumbnailViewer = new _pdf_thumbnail_viewer.PDFThumbnailViewer({
                    container: thumbnailContainer,
                    renderingQueue: pdfRenderingQueue,
                    linkService: pdfLinkService,
                    l10n: this.l10n
                  });
                  pdfRenderingQueue.setThumbnailViewer(this.pdfThumbnailViewer);
                }
                this.pdfHistory = new _pdf_history.PDFHistory({
                  linkService: pdfLinkService,
                  eventBus: eventBus
                });
                pdfLinkService.setHistory(this.pdfHistory);
                this.findBar = new _pdf_find_bar.PDFFindBar(appConfig.findBar, eventBus, this.l10n);
                this.pdfCursorTools = new _pdf_cursor_tools.PDFCursorTools({
                  container: container,
                  eventBus: eventBus,
                  cursorToolOnLoad: _app_options.AppOptions.get('cursorToolOnLoad')
                });
                this.toolbar = new _toolbar.Toolbar(appConfig.toolbar, eventBus, this.l10n);
                if (appConfig.secondaryToolbar) {
                  this.secondaryToolbar = new _secondary_toolbar.SecondaryToolbar(appConfig.secondaryToolbar, container, eventBus);
                }
                this.passwordPrompt = new _password_prompt.PasswordPrompt(appConfig.passwordOverlay, this.overlayManager, this.l10n);
                if (appConfig.sidebar) {
                  this.pdfOutlineViewer = new _pdf_outline_viewer.PDFOutlineViewer({
                    container: appConfig.sidebar.outlineView,
                    eventBus: eventBus,
                    linkService: pdfLinkService
                  });
                }
                if (appConfig.sidebar) {
                  sidebarConfig = Object.create(appConfig.sidebar);

                  sidebarConfig.pdfViewer = this.pdfViewer;
                  if (this.pdfThumbnailViewer) {
                    sidebarConfig.pdfThumbnailViewer = this.pdfThumbnailViewer;
                  }
                  this.pdfSidebar = new _pdf_sidebar.PDFSidebar(sidebarConfig, eventBus, this.l10n);
                  this.pdfSidebar.onToggled = this.forceRendering.bind(this);
                  this.pdfSidebarResizer = new _pdf_sidebar_resizer.PDFSidebarResizer(appConfig.sidebarResizer, eventBus, this.l10n);
                }

              case 26:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function _initializeViewerComponents() {
        return _ref5.apply(this, arguments);
      }

      return _initializeViewerComponents;
    }()
  }, {
    key: 'run',
    value: function run(config) {
      this.initialize(config).then(this.eventHandler.webViewerInitialized);
    }
  }, {
    key: 'zoomIn',
    value: function zoomIn(ticks) {
      var newScale = this.pdfViewer.currentScale;
      do {
        newScale = (newScale * DEFAULT_SCALE_DELTA).toFixed(2);
        newScale = Math.ceil(newScale * 10) / 10;
        newScale = Math.min(_ui_utils.MAX_SCALE, newScale);
      } while (--ticks > 0 && newScale < _ui_utils.MAX_SCALE);
      this.pdfViewer.currentScaleValue = newScale;
    }
  }, {
    key: 'zoomOut',
    value: function zoomOut(ticks) {
      var newScale = this.pdfViewer.currentScale;
      do {
        newScale = (newScale / DEFAULT_SCALE_DELTA).toFixed(2);
        newScale = Math.floor(newScale * 10) / 10;
        newScale = Math.max(_ui_utils.MIN_SCALE, newScale);
      } while (--ticks > 0 && newScale > _ui_utils.MIN_SCALE);
      this.pdfViewer.currentScaleValue = newScale;
    }
  }, {
    key: 'initPassiveLoading',
    value: function initPassiveLoading() {
      throw new Error('Not implemented: initPassiveLoading');
    }
  }, {
    key: 'setTitleUsingUrl',
    value: function setTitleUsingUrl() {
      var url = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    }
  }, {
    key: 'setTitle',
    value: function setTitle(title) {
      if (this.isViewerEmbedded) {}
    }
  }, {
    key: 'close',
    value: function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee6() {
        var errorWrapper, promise;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                errorWrapper = this.appConfig.errorWrapper.container;

                errorWrapper.setAttribute('hidden', 'true');

                if (this.pdfLoadingTask) {
                  _context6.next = 4;
                  break;
                }

                return _context6.abrupt('return');

              case 4:
                promise = this.pdfLoadingTask.destroy();

                this.pdfLoadingTask = null;
                if (this.pdfDocument) {
                  this.pdfDocument = null;
                  if (this.pdfThumbnailViewer) {
                    this.pdfThumbnailViewer.setDocument(null);
                  }
                  this.pdfViewer.setDocument(null);
                  this.pdfLinkService.setDocument(null);
                }
                this.store = null;
                this.isInitialViewSet = false;
                this.downloadComplete = false;
                this.url = '';
                this.baseUrl = '';
                this.contentDispositionFilename = null;
                if (this.pdfSidebar) {
                  this.pdfSidebar.reset();
                  this.pdfOutlineViewer.reset();
                }
                this.findBar.reset();
                this.toolbar.reset();
                if (this.secondaryToolbar) {
                  this.secondaryToolbar.reset();
                }
                if (typeof PDFBug !== 'undefined') {
                  PDFBug.cleanup();
                }
                return _context6.abrupt('return', promise);

              case 19:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function close() {
        return _ref6.apply(this, arguments);
      }

      return close;
    }()
  }, {
    key: 'open',
    value: function () {
      var _ref7 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee7(file, args) {
        var _this2 = this;

        var workerParameters, key, parameters, apiParameters, _key, prop, loadingTask;

        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                if (!this.pdfLoadingTask) {
                  _context7.next = 3;
                  break;
                }

                _context7.next = 3;
                return this.close();

              case 3:
                workerParameters = _app_options.AppOptions.getAll('worker');

                for (key in workerParameters) {
                  _pdf.GlobalWorkerOptions[key] = workerParameters[key];
                }
                parameters = Object.create(null);

                if (typeof file === 'string') {} else if (file && 'byteLength' in file) {
                  parameters.data = file;
                } else if (file.url && file.originalUrl) {}
                apiParameters = _app_options.AppOptions.getAll('api');

                for (_key in apiParameters) {
                  parameters[_key] = apiParameters[_key];
                }
                if (args) {
                  for (prop in args) {
                    parameters[prop] = args[prop];
                  }
                }
                loadingTask = (0, _pdf.getDocument)(parameters);

                this.pdfLoadingTask = loadingTask;
                loadingTask.onPassword = function (updateCallback, reason) {
                  _this2.passwordPrompt.setUpdateCallback(updateCallback, reason);
                  _this2.passwordPrompt.open();
                };
                loadingTask.onProgress = function (_ref8) {
                  var loaded = _ref8.loaded,
                      total = _ref8.total;

                  _this2.progress(loaded / total);
                };
                loadingTask.onUnsupportedFeature = this.fallback.bind(this);
                return _context7.abrupt('return', loadingTask.promise.then(function (pdfDocument) {
                  _this2.load(pdfDocument);
                }, function (exception) {
                  if (loadingTask !== _this2.pdfLoadingTask) {
                    return;
                  }
                  var message = exception && exception.message;
                  var loadingErrorMessage = void 0;
                  if (exception instanceof _pdf.InvalidPDFException) {
                    loadingErrorMessage = _this2.l10n.get('invalid_file_error', null, 'Invalid or corrupted PDF file.');
                  } else if (exception instanceof _pdf.MissingPDFException) {
                    loadingErrorMessage = _this2.l10n.get('missing_file_error', null, 'Missing PDF file.');
                  } else if (exception instanceof _pdf.UnexpectedResponseException) {
                    loadingErrorMessage = _this2.l10n.get('unexpected_response_error', null, 'Unexpected server response.');
                  } else {
                    loadingErrorMessage = _this2.l10n.get('loading_error', null, 'An error occurred while loading the PDF.');
                  }
                  return loadingErrorMessage.then(function (msg) {
                    _this2.error(msg, { message: message });
                    throw new Error(msg);
                  });
                }));

              case 16:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function open(_x3, _x4) {
        return _ref7.apply(this, arguments);
      }

      return open;
    }()
  }, {
    key: 'download',
    value: function download() {
      var _this3 = this;

      function downloadByUrl() {
        downloadManager.downloadUrl(url, filename);
      }
      var url = this.baseUrl;
      var filename = this.contentDispositionFilename || (0, _ui_utils.getPDFFileNameFromURL)(this.url);
      var downloadManager = this.downloadManager;
      downloadManager.onerror = function (err) {
        _this3.error('PDF failed to download: ' + err);
      };
      if (!this.pdfDocument || !this.downloadComplete) {
        downloadByUrl();
        return;
      }
      this.pdfDocument.getData().then(function (data) {
        var blob = new Blob([data], { type: 'application/pdf' });
        downloadManager.download(blob, url, filename);
      }).catch(downloadByUrl);
    }
  }, {
    key: 'fallback',
    value: function fallback(featureId) {}
  }, {
    key: 'error',
    value: function error(message, moreInfo) {
      var moreInfoText = [this.l10n.get('error_version_info', {
        version: _pdf.version || '?',
        build: _pdf.build || '?'
      }, 'PDF.js v{{version}} (build: {{build}})')];
      if (moreInfo) {
        moreInfoText.push(this.l10n.get('error_message', { message: moreInfo.message }, 'Message: {{message}}'));
        if (moreInfo.stack) {
          moreInfoText.push(this.l10n.get('error_stack', { stack: moreInfo.stack }, 'Stack: {{stack}}'));
        } else {
          if (moreInfo.filename) {
            moreInfoText.push(this.l10n.get('error_file', { file: moreInfo.filename }, 'File: {{file}}'));
          }
          if (moreInfo.lineNumber) {
            moreInfoText.push(this.l10n.get('error_line', { line: moreInfo.lineNumber }, 'Line: {{line}}'));
          }
        }
      }
      var errorWrapperConfig = this.appConfig.errorWrapper;
      var errorWrapper = errorWrapperConfig.container;
      errorWrapper.removeAttribute('hidden');
      var errorMessage = errorWrapperConfig.errorMessage;
      errorMessage.textContent = message;
      var closeButton = errorWrapperConfig.closeButton;
      closeButton.onclick = function () {
        errorWrapper.setAttribute('hidden', 'true');
      };
      var errorMoreInfo = errorWrapperConfig.errorMoreInfo;
      var moreInfoButton = errorWrapperConfig.moreInfoButton;
      var lessInfoButton = errorWrapperConfig.lessInfoButton;
      moreInfoButton.onclick = function () {
        errorMoreInfo.removeAttribute('hidden');
        moreInfoButton.setAttribute('hidden', 'true');
        lessInfoButton.removeAttribute('hidden');
        errorMoreInfo.style.height = errorMoreInfo.scrollHeight + 'px';
      };
      lessInfoButton.onclick = function () {
        errorMoreInfo.setAttribute('hidden', 'true');
        moreInfoButton.removeAttribute('hidden');
        lessInfoButton.setAttribute('hidden', 'true');
      };
      moreInfoButton.oncontextmenu = _ui_utils.noContextMenuHandler;
      lessInfoButton.oncontextmenu = _ui_utils.noContextMenuHandler;
      closeButton.oncontextmenu = _ui_utils.noContextMenuHandler;
      moreInfoButton.removeAttribute('hidden');
      lessInfoButton.setAttribute('hidden', 'true');
      Promise.all(moreInfoText).then(function (parts) {
        errorMoreInfo.value = parts.join('\n');
      });
    }
  }, {
    key: 'progress',
    value: function progress(level) {
      var _this4 = this;

      if (this.downloadComplete) {
        return;
      }
      var percent = Math.round(level * 100);
      if (percent > this.loadingBar.percent || isNaN(percent)) {
        this.loadingBar.percent = percent;
        var disableAutoFetch = this.pdfDocument ? this.pdfDocument.loadingParams['disableAutoFetch'] : _app_options.AppOptions.get('disableAutoFetch');
        if (disableAutoFetch && percent) {
          if (this.disableAutoFetchLoadingBarTimeout) {
            clearTimeout(this.disableAutoFetchLoadingBarTimeout);
            this.disableAutoFetchLoadingBarTimeout = null;
          }
          this.loadingBar.show();
          this.disableAutoFetchLoadingBarTimeout = setTimeout(function () {
            _this4.loadingBar.hide();
            _this4.disableAutoFetchLoadingBarTimeout = null;
          }, DISABLE_AUTO_FETCH_LOADING_BAR_TIMEOUT);
        }
      }
    }
  }, {
    key: 'load',
    value: function load(pdfDocument) {
      var _this5 = this;

      this.pdfDocument = pdfDocument;
      pdfDocument.getDownloadInfo().then(function () {
        _this5.downloadComplete = true;
        _this5.loadingBar.hide();
        firstPagePromise.then(function () {
          _this5.eventBus.dispatch('documentloaded', { source: _this5 });
          _this5.eventBus.dispatch('documentload', { source: _this5 });
        });
      });
      var pageModePromise = pdfDocument.getPageMode().catch(function () {});
      this.toolbar.setPagesCount(pdfDocument.numPages, false);
      if (this.secondaryToolbar) {
        this.secondaryToolbar.setPagesCount(pdfDocument.numPages);
      }
      var store = this.store = new _view_history.ViewHistory(pdfDocument.fingerprint);
      var baseDocumentUrl = void 0;
      baseDocumentUrl = null;
      this.pdfLinkService.setDocument(pdfDocument, baseDocumentUrl);
      var pdfViewer = this.pdfViewer;
      pdfViewer.setDocument(pdfDocument);
      var firstPagePromise = pdfViewer.firstPagePromise;
      var pagesPromise = pdfViewer.pagesPromise;
      var onePageRendered = pdfViewer.onePageRendered;
      if (this.pdfThumbnailViewer) {
        var pdfThumbnailViewer = this.pdfThumbnailViewer;
        pdfThumbnailViewer.setDocument(pdfDocument);
      }
      firstPagePromise.then(function (pdfPage) {
        _this5.loadingBar.setWidth(_this5.appConfig.viewerContainer);
        if (!_app_options.AppOptions.get('disableHistory') && !_this5.isViewerEmbedded) {
          var resetHistory = !_app_options.AppOptions.get('showPreviousViewOnLoad');
          _this5.pdfHistory.initialize(pdfDocument.fingerprint, resetHistory);
          if (_this5.pdfHistory.initialBookmark) {
            _this5.initialBookmark = _this5.pdfHistory.initialBookmark;
            _this5.initialRotation = _this5.pdfHistory.initialRotation;
          }
        }
        var storePromise = store.getMultiple({
          page: null,
          zoom: _ui_utils.DEFAULT_SCALE_VALUE,
          scrollLeft: '0',
          scrollTop: '0',
          rotation: null,
          sidebarView: _pdf_sidebar.SidebarView.NONE,
          scrollMode: null,
          spreadMode: null
        }).catch(function () {});
        Promise.all([storePromise, pageModePromise]).then(function () {
          var _ref10 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee8(_ref9) {
            var _ref11 = _slicedToArray(_ref9, 2),
                _ref11$ = _ref11[0],
                values = _ref11$ === undefined ? {} : _ref11$,
                pageMode = _ref11[1];

            var initialBookmark, zoom, hash, rotation, sidebarView, scrollMode, spreadMode;
            return _regenerator2.default.wrap(function _callee8$(_context8) {
              while (1) {
                switch (_context8.prev = _context8.next) {
                  case 0:
                    initialBookmark = _this5.initialBookmark;
                    zoom = _app_options.AppOptions.get('defaultZoomValue');
                    hash = zoom ? 'zoom=' + zoom : null;
                    rotation = null;
                    sidebarView = _app_options.AppOptions.get('sidebarViewOnLoad');
                    scrollMode = _app_options.AppOptions.get('scrollModeOnLoad');
                    spreadMode = _app_options.AppOptions.get('spreadModeOnLoad');

                    if (values.page && _app_options.AppOptions.get('showPreviousViewOnLoad')) {
                      hash = 'page=' + values.page + '&zoom=' + (zoom || values.zoom) + ',' + values.scrollLeft + ',' + values.scrollTop;
                      rotation = parseInt(values.rotation, 10);
                      sidebarView = sidebarView || values.sidebarView | 0;
                      scrollMode = scrollMode || values.scrollMode | 0;
                      spreadMode = spreadMode || values.spreadMode | 0;
                    }
                    if (pageMode && !_app_options.AppOptions.get('disablePageMode')) {
                      sidebarView = sidebarView || _this5.eventHandler.apiPageModeToSidebarView(pageMode);
                    }
                    _this5.setInitialView(hash, {
                      rotation: rotation,
                      sidebarView: sidebarView,
                      scrollMode: scrollMode,
                      spreadMode: spreadMode
                    });
                    _this5.eventBus.dispatch('documentinit', { source: _this5 });
                    if (!_this5.isViewerEmbedded) {
                      pdfViewer.focus();
                    }
                    _context8.next = 14;
                    return Promise.race([pagesPromise, new Promise(function (resolve) {
                      setTimeout(resolve, FORCE_PAGES_LOADED_TIMEOUT);
                    })]);

                  case 14:
                    if (!(!initialBookmark && !hash)) {
                      _context8.next = 16;
                      break;
                    }

                    return _context8.abrupt('return');

                  case 16:
                    if (!pdfViewer.hasEqualPageSizes) {
                      _context8.next = 18;
                      break;
                    }

                    return _context8.abrupt('return');

                  case 18:
                    _this5.initialBookmark = initialBookmark;
                    pdfViewer.currentScaleValue = pdfViewer.currentScaleValue;
                    _this5.setInitialView(hash);

                  case 21:
                  case 'end':
                    return _context8.stop();
                }
              }
            }, _callee8, _this5);
          }));

          return function (_x5) {
            return _ref10.apply(this, arguments);
          };
        }()).then(function () {
          pdfViewer.update();
        });
      });
      pdfDocument.getPageLabels().then(function (labels) {
        if (!labels || _app_options.AppOptions.get('disablePageLabels')) {
          return;
        }
        var i = 0,
            numLabels = labels.length;
        if (numLabels !== _this5.pagesCount) {
          console.error('The number of Page Labels does not match ' + 'the number of pages in the document.');
          return;
        }
        while (i < numLabels && labels[i] === (i + 1).toString()) {
          i++;
        }
        if (i === numLabels) {
          return;
        }
        pdfViewer.setPageLabels(labels);
        if (_this5.pdfThumbnailViewer) {
          _this5.pdfThumbnailViewer.setPageLabels(labels);
        }
        _this5.toolbar.setPagesCount(pdfDocument.numPages, true);
        _this5.toolbar.setPageNumber(pdfViewer.currentPageNumber, pdfViewer.currentPageLabel);
      });
      pagesPromise.then(function () {
        if (!_this5.supportsPrinting) {
          return;
        }
        pdfDocument.getJavaScript().then(function (javaScript) {
          if (!javaScript) {
            return;
          }
          javaScript.some(function (js) {
            if (!js) {
              return false;
            }
            console.warn('Warning: JavaScript is not supported');
            _this5.fallback(_pdf.UNSUPPORTED_FEATURES.javaScript);
            return true;
          });
          var regex = /\bprint\s*\(/;
          for (var i = 0, ii = javaScript.length; i < ii; i++) {
            var js = javaScript[i];
            if (js && regex.test(js)) {
              setTimeout(function () {
                window.print();
              });
              return;
            }
          }
        });
      });
      Promise.all([onePageRendered, _ui_utils.animationStarted]).then(function () {
        pdfDocument.getOutline().then(function (outline) {
          if (_this5.pdfOutlineViewer) {
            _this5.pdfOutlineViewer.render({ outline: outline });
          }
        });
        pdfDocument.getAttachments().then(function (attachments) {
          if (_this5.pdfAttachmentViewer) {
            _this5.pdfAttachmentViewer.render({ attachments: attachments });
          }
        });
      });
      pdfDocument.getMetadata().then(function (_ref12) {
        var info = _ref12.info,
            metadata = _ref12.metadata,
            contentDispositionFilename = _ref12.contentDispositionFilename;

        _this5.documentInfo = info;
        _this5.metadata = metadata;
        _this5.contentDispositionFilename = contentDispositionFilename;
        console.log('PDF ' + pdfDocument.fingerprint + ' [' + info.PDFFormatVersion + ' ' + (info.Producer || '-').trim() + ' / ' + (info.Creator || '-').trim() + ']' + ' (PDF.js: ' + (_pdf.version || '-') + (_app_options.AppOptions.get('enableWebGL') ? ' [WebGL]' : '') + ')');
        var pdfTitle = void 0;
        if (metadata && metadata.has('dc:title')) {
          var title = metadata.get('dc:title');
          if (title !== 'Untitled') {
            pdfTitle = title;
          }
        }
        if (!pdfTitle && info && info['Title']) {
          pdfTitle = info['Title'];
        }
        if (pdfTitle) {
          _this5.setTitle(pdfTitle + ' - ' + (contentDispositionFilename || document.title));
        } else if (contentDispositionFilename) {
          _this5.setTitle(contentDispositionFilename);
        }
        if (info.IsAcroFormPresent) {
          console.warn('Warning: AcroForm/XFA is not supported');
          _this5.fallback(_pdf.UNSUPPORTED_FEATURES.forms);
        }
      });
    }
  }, {
    key: 'setInitialView',
    value: function setInitialView(storedHash) {
      var _this6 = this;

      var _ref13 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          rotation = _ref13.rotation,
          sidebarView = _ref13.sidebarView,
          scrollMode = _ref13.scrollMode,
          spreadMode = _ref13.spreadMode;

      var setRotation = function setRotation(angle) {
        if ((0, _ui_utils.isValidRotation)(angle)) {
          _this6.pdfViewer.pagesRotation = angle;
        }
      };
      var setViewerModes = function setViewerModes(scroll, spread) {
        if (Number.isInteger(scroll)) {
          _this6.pdfViewer.scrollMode = scroll;
        }
        if (Number.isInteger(spread)) {
          _this6.pdfViewer.spreadMode = spread;
        }
      };
      setViewerModes(scrollMode, spreadMode);
      this.isInitialViewSet = true;
      if (this.pdfSidebar) {
        this.pdfSidebar.setInitialView(sidebarView);
      }
      if (this.initialBookmark) {
        setRotation(this.initialRotation);
        delete this.initialRotation;
        this.pdfLinkService.setHash(this.initialBookmark);
        this.initialBookmark = null;
      } else if (storedHash) {
        setRotation(rotation);
        this.pdfLinkService.setHash(storedHash);
      }
      this.toolbar.setPageNumber(this.pdfViewer.currentPageNumber, this.pdfViewer.currentPageLabel);
      if (this.secondaryToolbar) {
        this.secondaryToolbar.setPageNumber(this.pdfViewer.currentPageNumber);
      }
      if (!this.pdfViewer.currentScaleValue) {
        this.pdfViewer.currentScaleValue = _ui_utils.DEFAULT_SCALE_VALUE;
      }
    }
  }, {
    key: 'cleanup',
    value: function cleanup() {
      if (!this.pdfDocument) {
        return;
      }
      this.pdfViewer.cleanup();
      if (this.pdfThumbnailViewer) {
        this.pdfThumbnailViewer.cleanup();
      }
      if (this.pdfViewer.renderer !== _ui_utils.RendererType.SVG) {
        this.pdfDocument.cleanup();
      }
    }
  }, {
    key: 'forceRendering',
    value: function forceRendering() {
      this.pdfRenderingQueue.printing = this.printing;
      if (this.pdfSidebar) {
        this.pdfRenderingQueue.isThumbnailViewEnabled = this.pdfSidebar.isThumbnailViewVisible;
        this.pdfRenderingQueue.renderHighestPriority();
      }
    }
  }, {
    key: 'beforePrint',
    value: function beforePrint() {
      var _this7 = this;

      if (this.printService) {
        return;
      }
      if (!this.supportsPrinting) {
        this.l10n.get('printing_not_supported', null, 'Warning: Printing is not fully supported by ' + 'this browser.').then(function (printMessage) {
          _this7.error(printMessage);
        });
        return;
      }
      if (!this.pdfViewer.pageViewsReady) {
        this.l10n.get('printing_not_ready', null, 'Warning: The PDF is not fully loaded for printing.').then(function (notReadyMessage) {
          window.alert(notReadyMessage);
        });
        return;
      }
      var pagesOverview = this.pdfViewer.getPagesOverview();
      var printContainer = this.appConfig.printContainer;
      var printService = PDFPrintServiceFactory.instance.createPrintService(this.pdfDocument, pagesOverview, printContainer, this.l10n);
      this.printService = printService;
      this.forceRendering();
      printService.layout();
    }
  }, {
    key: 'afterPrint',
    value: function afterPrint() {
      if (this.printService) {
        this.printService.destroy();
        this.printService = null;
      }
      this.forceRendering();
    }
  }, {
    key: 'rotatePages',
    value: function rotatePages(delta) {
      if (!this.pdfDocument) {
        return;
      }
      var newRotation = (this.pdfViewer.pagesRotation + 360 + delta) % 360;
      this.pdfViewer.pagesRotation = newRotation;
    }
  }, {
    key: 'requestPresentationMode',
    value: function requestPresentationMode() {
      if (!this.pdfPresentationMode) {
        return;
      }
      this.pdfPresentationMode.request();
    }
  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      var eventBus = this.eventBus,
          _boundEvents = this._boundEvents;

      _boundEvents.beforePrint = this.beforePrint.bind(this);
      _boundEvents.afterPrint = this.afterPrint.bind(this);
      eventBus.on('resize', this.eventHandler.webViewerResize);
      eventBus.on('hashchange', this.eventHandler.webViewerHashchange);
      eventBus.on('beforeprint', _boundEvents.beforePrint);
      eventBus.on('afterprint', _boundEvents.afterPrint);
      eventBus.on('pagerendered', this.eventHandler.webViewerPageRendered);
      eventBus.on('textlayerrendered', this.eventHandler.webViewerTextLayerRendered);
      eventBus.on('updateviewarea', this.eventHandler.webViewerUpdateViewarea);
      eventBus.on('pagechanging', this.eventHandler.webViewerPageChanging);
      eventBus.on('scalechanging', this.eventHandler.webViewerScaleChanging);
      eventBus.on('rotationchanging', this.eventHandler.webViewerRotationChanging);
      eventBus.on('sidebarviewchanged', this.eventHandler.webViewerSidebarViewChanged);
      eventBus.on('pagemode', this.eventHandler.webViewerPageMode);
      eventBus.on('namedaction', this.eventHandler.webViewerNamedAction);
      eventBus.on('openfile', this.eventHandler.webViewerOpenFile);
      eventBus.on('firstpage', this.eventHandler.webViewerFirstPage);
      eventBus.on('lastpage', this.eventHandler.webViewerLastPage);
      eventBus.on('nextpage', this.eventHandler.webViewerNextPage);
      eventBus.on('previouspage', this.eventHandler.webViewerPreviousPage);
      eventBus.on('zoomin', this.eventHandler.webViewerZoomIn);
      eventBus.on('zoomout', this.eventHandler.webViewerZoomOut);
      eventBus.on('pagenumberchanged', this.eventHandler.webViewerPageNumberChanged);
      eventBus.on('scalechanged', this.eventHandler.webViewerScaleChanged);
      eventBus.on('rotatecw', this.eventHandler.webViewerRotateCw);
      eventBus.on('rotateccw', this.eventHandler.webViewerRotateCcw);
      eventBus.on('switchscrollmode', this.eventHandler.webViewerSwitchScrollMode);
      eventBus.on('scrollmodechanged', this.eventHandler.webViewerScrollModeChanged);
      eventBus.on('switchspreadmode', this.eventHandler.webViewerSwitchSpreadMode);
      eventBus.on('spreadmodechanged', this.eventHandler.webViewerSpreadModeChanged);
      eventBus.on('find', this.eventHandler.webViewerFind);
      eventBus.on('findfromurlhash', this.eventHandler.webViewerFindFromUrlHash);
      eventBus.on('updatefindmatchescount', this.eventHandler.webViewerUpdateFindMatchesCount);
      eventBus.on('updatefindcontrolstate', this.eventHandler.webViewerUpdateFindControlState);
      eventBus.on('cursortoolchanged', this.eventHandler.webViewerCursorTooleChange);
      eventBus.on('fileinputchange', this.eventHandler.webViewerFileInputChange);
    }
  }, {
    key: 'bindWindowEvents',
    value: function bindWindowEvents() {
      var eventBus = this.eventBus,
          _boundEvents = this._boundEvents;

      _boundEvents.windowResize = function () {
        eventBus.dispatch('resize', { source: window });
      };
      _boundEvents.windowHashChange = function () {
        eventBus.dispatch('hashchange', {
          source: window,
          hash: document.location.hash.substring(1)
        });
      };
      _boundEvents.windowBeforePrint = function () {
        eventBus.dispatch('beforeprint', { source: window });
      };
      _boundEvents.windowAfterPrint = function () {
        eventBus.dispatch('afterprint', { source: window });
      };
      this.appConfig.viewerContainer.addEventListener('wheel', this.eventHandler.webViewerWheel);
      window.addEventListener('click', this.eventHandler.webViewerClick);
      window.addEventListener('keydown', this.eventHandler.webViewerKeyDown);
      window.addEventListener('resize', _boundEvents.windowResize);
      window.addEventListener('hashchange', _boundEvents.windowHashChange);
      window.addEventListener('beforeprint', _boundEvents.windowBeforePrint);
      window.addEventListener('afterprint', _boundEvents.windowAfterPrint);
    }
  }, {
    key: 'unbindEvents',
    value: function unbindEvents() {
      var eventBus = this.eventBus,
          _boundEvents = this._boundEvents;

      eventBus.off('resize', this.eventHandler.webViewerResize);
      eventBus.off('hashchange', this.eventHandler.webViewerHashchange);
      eventBus.off('beforeprint', _boundEvents.beforePrint);
      eventBus.off('afterprint', _boundEvents.afterPrint);
      eventBus.off('pagerendered', this.eventHandler.webViewerPageRendered);
      eventBus.off('textlayerrendered', this.eventHandler.webViewerTextLayerRendered);
      eventBus.off('updateviewarea', this.eventHandler.webViewerUpdateViewarea);
      eventBus.off('pagechanging', this.eventHandler.webViewerPageChanging);
      eventBus.off('scalechanging', this.eventHandler.webViewerScaleChanging);
      eventBus.off('rotationchanging', this.eventHandler.webViewerRotationChanging);
      eventBus.off('sidebarviewchanged', this.eventHandler.webViewerSidebarViewChanged);
      eventBus.off('pagemode', this.eventHandler.webViewerPageMode);
      eventBus.off('namedaction', this.eventHandler.webViewerNamedAction);
      eventBus.off('openfile', this.eventHandler.webViewerOpenFile);
      eventBus.off('print', this.eventHandler.webViewerPrint);
      eventBus.off('firstpage', this.eventHandler.webViewerFirstPage);
      eventBus.off('lastpage', this.eventHandler.webViewerLastPage);
      eventBus.off('nextpage', this.eventHandler.webViewerNextPage);
      eventBus.off('previouspage', this.eventHandler.webViewerPreviousPage);
      eventBus.off('zoomin', this.eventHandler.webViewerZoomIn);
      eventBus.off('zoomout', this.eventHandler.webViewerZoomOut);
      eventBus.off('pagenumberchanged', this.eventHandler.webViewerPageNumberChanged);
      eventBus.off('scalechanged', this.eventHandler.webViewerScaleChanged);
      eventBus.off('rotatecw', this.eventHandler.webViewerRotateCw);
      eventBus.off('rotateccw', this.eventHandler.webViewerRotateCcw);
      eventBus.off('switchscrollmode', this.eventHandler.webViewerSwitchScrollMode);
      eventBus.off('scrollmodechanged', this.eventHandler.webViewerScrollModeChanged);
      eventBus.off('switchspreadmode', this.eventHandler.webViewerSwitchSpreadMode);
      eventBus.off('spreadmodechanged', this.eventHandler.webViewerSpreadModeChanged);
      eventBus.off('find', this.eventHandler.webViewerFind);
      eventBus.off('findfromurlhash', this.eventHandler.webViewerFindFromUrlHash);
      eventBus.off('updatefindmatchescount', this.eventHandler.webViewerUpdateFindMatchesCount);
      eventBus.off('updatefindcontrolstate', this.eventHandler.webViewerUpdateFindControlState);
      eventBus.off('cursortoolchanged', this.eventHandler.webViewerCursorTooleChange);
      eventBus.off('fileinputchange', this.eventHandler.webViewerFileInputChange);
      _boundEvents.beforePrint = null;
      _boundEvents.afterPrint = null;
    }
  }, {
    key: 'unbindWindowEvents',
    value: function unbindWindowEvents() {
      var _boundEvents = this._boundEvents;

      this.appConfig.viewerContainer.removeEventListener('wheel', this.eventHandler.webViewerWheel);
      window.removeEventListener('click', this.eventHandler.webViewerClick);
      window.removeEventListener('keydown', this.eventHandler.webViewerKeyDown);
      window.removeEventListener('resize', _boundEvents.windowResize);
      window.removeEventListener('hashchange', _boundEvents.windowHashChange);
      window.removeEventListener('beforeprint', _boundEvents.windowBeforePrint);
      window.removeEventListener('afterprint', _boundEvents.windowAfterPrint);
      _boundEvents.windowResize = null;
      _boundEvents.windowHashChange = null;
      _boundEvents.windowBeforePrint = null;
      _boundEvents.windowAfterPrint = null;
    }
  }, {
    key: 'pagesCount',
    get: function get() {
      return this.pdfDocument ? this.pdfDocument.numPages : 0;
    }
  }, {
    key: 'page',
    set: function set(val) {
      this.pdfViewer.currentPageNumber = val;
    },
    get: function get() {
      return this.pdfViewer.currentPageNumber;
    }
  }, {
    key: 'printing',
    get: function get() {
      return !!this.printService;
    }
  }, {
    key: 'supportsPrinting',
    get: function get() {
      return PDFPrintServiceFactory.instance.supportsPrinting;
    }
  }, {
    key: 'supportsIntegratedFind',
    get: function get() {
      return PDFViewerApplication.externalServices.supportsIntegratedFind;
    }
  }, {
    key: 'supportsDocumentFonts',
    get: function get() {
      return PDFViewerApplication.externalServices.supportsDocumentFonts;
    }
  }, {
    key: 'supportsDocumentColors',
    get: function get() {
      return PDFViewerApplication.externalServices.supportsDocumentColors;
    }
  }, {
    key: 'loadingBar',
    get: function get() {
      var bar = new _ui_utils.ProgressBar('#loadingBar');
      return (0, _pdf.shadow)(this, 'loadingBar', bar);
    }
  }, {
    key: 'supportedMouseWheelZoomModifierKeys',
    get: function get() {
      return PDFViewerApplication.externalServices.supportedMouseWheelZoomModifierKeys;
    }
  }, {
    key: 'appOptions',
    get: function get() {
      return _app_options.AppOptions;
    }
  }]);

  return PDFViewerApplication;
}();

PDFViewerApplication.externalServices = DefaultExternalServices;

var PDFViewerApplicationEventHandler = function PDFViewerApplicationEventHandler(pdfViewerApplication) {
  var _this8 = this;

  _classCallCheck(this, PDFViewerApplicationEventHandler);

  this.pdfViewerApplication = pdfViewerApplication;
  this.webViewerFileInputChange = undefined;
  this.webViewerFileInputChange = function webViewerFileInputChange(evt) {
    if (this.pdfViewerApplication.pdfViewer && this.pdfViewerApplication.pdfViewer.isInPresentationMode) {
      return;
    }
    var file = evt.fileInput.files[0];
    if (_pdf.URL.createObjectURL && !_app_options.AppOptions.get('disableCreateObjectURL')) {
      var _url = _pdf.URL.createObjectURL(file);
      if (file.name) {
        _url = {
          url: _url,
          originalUrl: file.name
        };
      }
      this.pdfViewerApplication.open(_url);
    } else {
      this.pdfViewerApplication.setTitleUsingUrl(file.name);
      var fileReader = new FileReader();
      fileReader.onload = function webViewerChangeFileReaderOnload(evt) {
        var buffer = evt.target.result;
        this.pdfViewerApplication.open(new Uint8Array(buffer));
      };
      fileReader.readAsArrayBuffer(file);
    }
    var appConfig = this.pdfViewerApplication.appConfig;
    if (appConfig.secondaryToolbar) {
      appConfig.secondaryToolbar.viewBookmarkButton.setAttribute('hidden', 'true');
      appConfig.secondaryToolbar.downloadButton.setAttribute('hidden', 'true');
    }
  };
  this.zoomDisabled = false;
  this.zoomDisabledTimeout = undefined;
  this.webViewerInitialized = function () {
    var appConfig = _this8.pdfViewerApplication.appConfig;
    var eventBus = _this8.pdfViewerApplication.eventBus;
    var pdfViewerApplication = _this8.pdfViewerApplication;
    var file = void 0;
    var queryString = document.location.search.substring(1);
    var params = (0, _ui_utils.parseQueryString)(queryString);
    file = 'file' in params ? params.file : _app_options.AppOptions.get('defaultUrl');
    var fileInput = document.createElement('input');
    fileInput.id = appConfig.openFileInputName;
    fileInput.className = 'fileInput';
    fileInput.setAttribute('type', 'file');
    fileInput.oncontextmenu = _ui_utils.noContextMenuHandler;
    document.body.appendChild(fileInput);
    if (!window.File || !window.FileReader || !window.FileList || !window.Blob) {
      appConfig.toolbar.openFile.setAttribute('hidden', 'true');
      if (appConfig.secondaryToolbar) {
        appConfig.secondaryToolbar.openFileButton.setAttribute('hidden', 'true');
      }
    } else {
      fileInput.value = null;
    }
    fileInput.addEventListener('change', function (evt) {
      var files = evt.target.files;
      if (!files || files.length === 0) {
        return;
      }
      eventBus.dispatch('fileinputchange', {
        source: this,
        fileInput: evt.target
      });
    });
    appConfig.mainContainer.addEventListener('dragover', function (evt) {
      evt.preventDefault();
      evt.dataTransfer.dropEffect = 'move';
    });
    appConfig.mainContainer.addEventListener('drop', function (evt) {
      evt.preventDefault();
      var files = evt.dataTransfer.files;
      if (!files || files.length === 0) {
        return;
      }
      eventBus.dispatch('fileinputchange', {
        source: this,
        fileInput: evt.dataTransfer
      });
    });
    if (!_this8.pdfViewerApplication.supportsPrinting) {
      appConfig.toolbar.print.classList.add('hidden');
      if (appConfig.secondaryToolbar) {
        appConfig.secondaryToolbar.printButton.classList.add('hidden');
      }
    }
    if (_this8.pdfViewerApplication.supportsIntegratedFind) {
      appConfig.toolbar.viewFind.classList.add('hidden');
    }
    appConfig.mainContainer.addEventListener('transitionend', function (evt) {
      if (evt.target === this) {
        eventBus.dispatch('resize', { source: this });
      }
    }, true);
    try {
      _this8.webViewerOpenFileViaURLFunc(file);
    } catch (reason) {
      _this8.pdfViewerApplication.l10n.get('loading_error', null, 'An error occurred while loading the PDF.').then(function (msg) {
        _this8.pdfViewerApplication.error(msg, reason);
      });
    }
  };
  this.loadAndEnablePDFBug = function (enabledTabs) {
    var appConfig = _this8.pdfViewerApplication.appConfig;
    return (0, _pdf.loadScript)(appConfig.debuggerScriptPath).then(function () {
      PDFBug.enable(enabledTabs);
      PDFBug.init({
        OPS: _pdf.OPS,
        createObjectURL: _pdf.createObjectURL
      }, appConfig.mainContainer);
    });
  };
  this.webViewerOpenFileViaURLFunc = function (file) {
    var webViewerOpenFileViaURL = void 0;
    if (_this8.pdfViewerApplication.pdfFileLoader) {
      webViewerOpenFileViaURL = _this8.pdfViewerApplication.pdfFileLoader.webViewerOpenFileViaURL;
    } else {
      webViewerOpenFileViaURL = function webViewerOpenFileViaURL(file) {
        if (file && file.lastIndexOf('file:', 0) === 0) {
          this.pdfViewerApplication.setTitleUsingUrl(file);
          var xhr = new XMLHttpRequest();
          xhr.onload = function () {
            this.pdfViewerApplication.open(new Uint8Array(xhr.response));
          };
          try {
            xhr.open('GET', file);
            xhr.responseType = 'arraybuffer';
            xhr.send();
          } catch (ex) {
            throw ex;
          }
          return;
        }
        if (file) {
          this.pdfViewerApplication.open(file);
        }
      };
    }
    webViewerOpenFileViaURL(file);
  };
  this.webViewerPageRendered = function (evt) {
    var pageNumber = evt.pageNumber;
    var pageIndex = pageNumber - 1;
    var pageView = _this8.pdfViewerApplication.pdfViewer.getPageView(pageIndex);
    if (pageNumber === _this8.pdfViewerApplication.page) {
      _this8.pdfViewerApplication.toolbar.updateLoadingIndicatorState(false);
    }
    if (!pageView) {
      return;
    }
    if (_this8.pdfViewerApplication.pdfSidebar) {
      if (_this8.pdfViewerApplication.pdfSidebar.isThumbnailViewVisible) {
        var thumbnailView = _this8.pdfViewerApplication.pdfThumbnailViewer.getThumbnail(pageIndex);
        thumbnailView.setImage(pageView);
      }
    }
    if (typeof Stats !== 'undefined' && Stats.enabled && pageView.stats) {
      Stats.add(pageNumber, pageView.stats);
    }
    if (pageView.error) {
      _this8.pdfViewerApplication.l10n.get('rendering_error', null, 'An error occurred while rendering the page.').then(function (msg) {
        _this8.pdfViewerApplication.error(msg, pageView.error);
      });
    }
  };
  this.webViewerTextLayerRendered = function (evt) {};
  this.webViewerPageMode = function (evt) {
    var mode = evt.mode,
        view = void 0;
    switch (mode) {
      case 'thumbs':
        view = _pdf_sidebar.SidebarView.THUMBS;
        break;
      case 'bookmarks':
      case 'outline':
        view = _pdf_sidebar.SidebarView.OUTLINE;
        break;
      case 'attachments':
        view = _pdf_sidebar.SidebarView.ATTACHMENTS;
        break;
      case 'none':
        view = _pdf_sidebar.SidebarView.NONE;
        break;
      default:
        console.error('Invalid "pagemode" hash parameter: ' + mode);
        return;
    }
    if (_this8.pdfViewerApplication.pdfSidebar) {
      _this8.pdfViewerApplication.pdfSidebar.switchView(view, true);
    }
  };
  this.webViewerNamedAction = function (evt) {
    var action = evt.action;
    switch (action) {
      case 'GoToPage':
        _this8.pdfViewerApplication.appConfig.toolbar.pageNumber.select();
        break;
      case 'Find':
        if (!_this8.pdfViewerApplication.supportsIntegratedFind) {
          _this8.pdfViewerApplication.findBar.toggle();
        }
        break;
    }
  };
  this.webViewerPresentationModeChanged = function (evt) {
    var active = evt.active,
        switchInProgress = evt.switchInProgress;

    _this8.pdfViewerApplication.pdfViewer.presentationModeState = switchInProgress ? _ui_utils.PresentationModeState.CHANGING : active ? _ui_utils.PresentationModeState.FULLSCREEN : _ui_utils.PresentationModeState.NORMAL;
  };
  this.webViewerSidebarViewChanged = function (evt) {
    _this8.pdfViewerApplication.pdfRenderingQueue.isThumbnailViewEnabled = _this8.pdfViewerApplication.pdfSidebar.isThumbnailViewVisible;
    var store = _this8.pdfViewerApplication.store;
    if (store && _this8.pdfViewerApplication.isInitialViewSet) {
      store.set('sidebarView', evt.view).catch(function () {});
    }
  };
  this.webViewerUpdateViewarea = function (evt) {
    var location = evt.location,
        store = _this8.pdfViewerApplication.store;
    if (store && _this8.pdfViewerApplication.isInitialViewSet) {
      store.setMultiple({
        'page': location.pageNumber,
        'zoom': location.scale,
        'scrollLeft': location.left,
        'scrollTop': location.top,
        'rotation': location.rotation
      }).catch(function () {});
    }
    var href = _this8.pdfViewerApplication.pdfLinkService.getAnchorUrl(location.pdfOpenParams);
    if (_this8.pdfViewerApplication.appConfig.secondaryToolbar) {
      _this8.pdfViewerApplication.appConfig.secondaryToolbar.viewBookmarkButton.href = href;
    }
    var currentPage = _this8.pdfViewerApplication.pdfViewer.getPageView(_this8.pdfViewerApplication.page - 1);
    var loading = currentPage.renderingState !== _pdf_rendering_queue.RenderingStates.FINISHED;
    _this8.pdfViewerApplication.toolbar.updateLoadingIndicatorState(loading);
  };
  this.webViewerScrollModeChanged = function (evt) {
    var store = _this8.pdfViewerApplication.store;
    if (store && _this8.pdfViewerApplication.isInitialViewSet) {
      store.set('scrollMode', evt.mode).catch(function () {});
    }
  };
  this.webViewerSpreadModeChanged = function (evt) {
    var store = _this8.pdfViewerApplication.store;
    if (store && _this8.pdfViewerApplication.isInitialViewSet) {
      store.set('spreadMode', evt.mode).catch(function () {});
    }
  };
  this.webViewerResize = function () {
    var _pdfViewerApplication = _this8.pdfViewerApplication,
        pdfDocument = _pdfViewerApplication.pdfDocument,
        pdfViewer = _pdfViewerApplication.pdfViewer;

    if (!pdfDocument) {
      return;
    }
    var currentScaleValue = pdfViewer.currentScaleValue;
    if (currentScaleValue === 'auto' || currentScaleValue === 'page-fit' || currentScaleValue === 'page-width') {
      pdfViewer.currentScaleValue = currentScaleValue;
    }
    pdfViewer.update();
  };
  this.webViewerHashchange = function (evt) {
    var hash = evt.hash;
    if (!hash) {
      return;
    }
    if (!_this8.pdfViewerApplication.isInitialViewSet) {
      _this8.pdfViewerApplication.initialBookmark = hash;
    } else if (!_this8.pdfViewerApplication.pdfHistory.popStateInProgress) {
      _this8.pdfViewerApplication.pdfLinkService.setHash(hash);
    }
  };
  this.webViewerPresentationMode = function () {
    _this8.pdfViewerApplication.requestPresentationMode();
  };
  this.webViewerOpenFile = function () {
    var openFileInputName = _this8.pdfViewerApplication.appConfig.openFileInputName;
    document.getElementById(openFileInputName).click();
  };
  this.webViewerPrint = function () {
    window.print();
  };
  this.webViewerFirstPage = function () {
    if (_this8.pdfViewerApplication.pdfDocument) {
      _this8.pdfViewerApplication.page = 1;
    }
  };
  this.webViewerLastPage = function () {
    if (_this8.pdfViewerApplication.pdfDocument) {
      _this8.pdfViewerApplication.page = _this8.pdfViewerApplication.pagesCount;
    }
  };
  this.webViewerNextPage = function () {
    _this8.pdfViewerApplication.page++;
  };
  this.webViewerPreviousPage = function () {
    _this8.pdfViewerApplication.page--;
  };
  this.webViewerZoomIn = function () {
    _this8.pdfViewerApplication.zoomIn();
  };
  this.webViewerZoomOut = function () {
    _this8.pdfViewerApplication.zoomOut();
  };
  this.webViewerCursorTooleChange = function (evt) {
    _this8.pdfViewerApplication.appConfig.toolbar.cursorSelectToolButton.classList.remove('toggled');
    _this8.pdfViewerApplication.appConfig.toolbar.cursorHandToolButton.classList.remove('toggled');
    switch (evt.tool) {
      case _pdf_cursor_tools.CursorTool.SELECT:
        _this8.pdfViewerApplication.appConfig.toolbar.cursorSelectToolButton.classList.add('toggled');
        break;
      case _pdf_cursor_tools.CursorTool.HAND:
        _this8.pdfViewerApplication.appConfig.toolbar.cursorHandToolButton.classList.add('toggled');
        break;
    }
  };
  this.webViewerPageNumberChanged = function (evt) {
    var pdfViewer = _this8.pdfViewerApplication.pdfViewer;
    pdfViewer.currentPageLabel = evt.value;
    if (evt.value !== pdfViewer.currentPageNumber.toString() && evt.value !== pdfViewer.currentPageLabel) {
      _this8.pdfViewerApplication.toolbar.setPageNumber(pdfViewer.currentPageNumber, pdfViewer.currentPageLabel);
    }
  };
  this.webViewerScaleChanged = function (evt) {
    _this8.pdfViewerApplication.pdfViewer.currentScaleValue = evt.value;
  };
  this.webViewerRotateCw = function () {
    _this8.pdfViewerApplication.rotatePages(90);
  };
  this.webViewerRotateCcw = function () {
    _this8.pdfViewerApplication.rotatePages(-90);
  };
  this.webViewerSwitchScrollMode = function (evt) {
    _this8.pdfViewerApplication.pdfViewer.scrollMode = evt.mode;
  };
  this.webViewerSwitchSpreadMode = function (evt) {
    _this8.pdfViewerApplication.pdfViewer.spreadMode = evt.mode;
  };
  this.webViewerFind = function (evt) {
    _this8.pdfViewerApplication.findController.executeCommand('find' + evt.type, {
      query: evt.query,
      phraseSearch: evt.phraseSearch,
      caseSensitive: evt.caseSensitive,
      entireWord: evt.entireWord,
      highlightAll: evt.highlightAll,
      findPrevious: evt.findPrevious
    });
  };
  this.webViewerFindFromUrlHash = function (evt) {
    _this8.pdfViewerApplication.findController.executeCommand('find', {
      query: evt.query,
      phraseSearch: evt.phraseSearch,
      caseSensitive: false,
      entireWord: false,
      highlightAll: true,
      findPrevious: false
    });
  };
  this.webViewerUpdateFindMatchesCount = function (_ref14) {
    var matchesCount = _ref14.matchesCount;

    if (_this8.pdfViewerApplication.supportsIntegratedFind) {
      _this8.pdfViewerApplication.externalServices.updateFindMatchesCount(matchesCount);
    } else {
      _this8.pdfViewerApplication.findBar.updateResultsCount(matchesCount);
    }
  };
  this.webViewerUpdateFindControlState = function (_ref15) {
    var state = _ref15.state,
        previous = _ref15.previous,
        matchesCount = _ref15.matchesCount;

    if (_this8.pdfViewerApplication.supportsIntegratedFind) {
      _this8.pdfViewerApplication.externalServices.updateFindControlState({
        result: state,
        findPrevious: previous,
        matchesCount: matchesCount
      });
    } else {
      _this8.pdfViewerApplication.findBar.updateUIState(state, previous, matchesCount);
    }
  };
  this.webViewerScaleChanging = function (evt) {
    _this8.pdfViewerApplication.toolbar.setPageScale(evt.presetValue, evt.scale);
    _this8.pdfViewerApplication.pdfViewer.update();
  };
  this.webViewerRotationChanging = function (evt) {
    if (_this8.pdfViewerApplication.pdfThumbnailViewer) {
      _this8.pdfViewerApplication.pdfThumbnailViewer.pagesRotation = evt.pagesRotation;
    }
    _this8.pdfViewerApplication.forceRendering();
    _this8.pdfViewerApplication.pdfViewer.currentPageNumber = evt.pageNumber;
  };
  this.webViewerPageChanging = function (evt) {
    var page = evt.pageNumber;
    _this8.pdfViewerApplication.toolbar.setPageNumber(page, evt.pageLabel || null);
    if (_this8.pdfViewerApplication.secondaryToolbar) {
      _this8.pdfViewerApplication.secondaryToolbar.setPageNumber(page);
    }
    if (_this8.pdfViewerApplication.pdfSidebar) {
      if (_this8.pdfViewerApplication.pdfSidebar.isThumbnailViewVisible) {
        _this8.pdfViewerApplication.pdfThumbnailViewer.scrollThumbnailIntoView(page);
      }
    }
    if (typeof Stats !== 'undefined' && Stats.enabled) {
      var pageView = _this8.pdfViewerApplication.pdfViewer.getPageView(page - 1);
      if (pageView && pageView.stats) {
        Stats.add(page, pageView.stats);
      }
    }
  };
  this.webViewerWheel = function (evt) {
    if (evt.target === document.body) {
      return;
    }
    var pdfViewer = _this8.pdfViewerApplication.pdfViewer;
    if (pdfViewer.isInPresentationMode) {
      return;
    }
    if (evt.ctrlKey || evt.metaKey) {
      _this8.zoomDisabled = true;
      _this8.zoomDisabledTimeout = setTimeout(function () {
        this.zoomDisabled = false;
      }, 1000);
      clearTimeout(_this8.zoomDisabledTimeout);
    } else {
      var support = _this8.pdfViewerApplication.supportedMouseWheelZoomModifierKeys;
      if (evt.ctrlKey && !support.ctrlKey || evt.metaKey && !support.metaKey) {
        return;
      }
      evt.preventDefault();
      _this8.zoomDisabled = false;
      if (_this8.zoomDisabled) {
        return;
      }
      var previousScale = pdfViewer.currentScale;
      var delta = (0, _ui_utils.normalizeWheelEventDelta)(evt);
      var MOUSE_WHEEL_DELTA_PER_PAGE_SCALE = 3.0;
      var ticks = delta * MOUSE_WHEEL_DELTA_PER_PAGE_SCALE;
      if (ticks < 0) {
        _this8.pdfViewerApplication.zoomOut(-ticks);
      } else {
        _this8.pdfViewerApplication.zoomIn(ticks);
      }
      var currentScale = pdfViewer.currentScale;
      if (previousScale !== currentScale) {
        var scaleCorrectionFactor = currentScale / previousScale - 1;
        var rect = pdfViewer.container.getBoundingClientRect();
        var dx = evt.clientX - rect.left;
        var dy = evt.clientY - rect.top;
        pdfViewer.container.scrollLeft += dx * scaleCorrectionFactor;
        pdfViewer.container.scrollTop += dy * scaleCorrectionFactor;
      }
    }
  };
  this.webViewerClick = function (evt) {
    if (_this8.pdfViewerApplication.secondaryToolbar && !_this8.pdfViewerApplication.secondaryToolbar.isOpen) {
      return;
    }
    var appConfig = _this8.pdfViewerApplication.appConfig;
    if (appConfig.secondaryToolbar) {
      if (_this8.pdfViewerApplication.pdfViewer.containsElement(evt.target) || appConfig.toolbar.container.contains(evt.target) && evt.target !== appConfig.secondaryToolbar.toggleButton) {
        _this8.pdfViewerApplication.secondaryToolbar.close();
      }
    }
    if (evt.target === appConfig.toolbar.cursorSelectToolButton) {
      _this8.pdfViewerApplication.appConfig.toolbar.cursorSelectToolButton.classList.add('toggled');
      _this8.pdfViewerApplication.pdfCursorTools.switchTool(_pdf_cursor_tools.CursorTool.SELECT);
    }
    if (evt.target === appConfig.toolbar.cursorHandToolButton) {
      _this8.pdfViewerApplication.appConfig.toolbar.cursorHandToolButton.classList.add('toggled');
      _this8.pdfViewerApplication.pdfCursorTools.switchTool(_pdf_cursor_tools.CursorTool.HAND);
    }
  };
  this.webViewerKeyDown = function (evt) {
    if (_this8.pdfViewerApplication.overlayManager.active) {
      return;
    }
    var handled = false,
        ensureViewerFocused = false;
    var cmd = (evt.ctrlKey ? 1 : 0) | (evt.altKey ? 2 : 0) | (evt.shiftKey ? 4 : 0) | (evt.metaKey ? 8 : 0);
    var pdfViewer = _this8.pdfViewerApplication.pdfViewer;
    var isViewerInPresentationMode = pdfViewer && pdfViewer.isInPresentationMode;
    if (cmd === 1 || cmd === 8 || cmd === 5 || cmd === 12) {
      switch (evt.keyCode) {
        case 70:
          if (!_this8.pdfViewerApplication.supportsIntegratedFind) {
            _this8.pdfViewerApplication.findBar.open();
            handled = true;
          }
          break;
        case 71:
          if (!_this8.pdfViewerApplication.supportsIntegratedFind) {
            var findState = _this8.pdfViewerApplication.findController.state;
            if (findState) {
              _this8.pdfViewerApplication.findController.executeCommand('findagain', {
                query: findState.query,
                phraseSearch: findState.phraseSearch,
                caseSensitive: findState.caseSensitive,
                entireWord: findState.entireWord,
                highlightAll: findState.highlightAll,
                findPrevious: cmd === 5 || cmd === 12
              });
            }
            handled = true;
          }
          break;
        case 61:
        case 107:
        case 187:
        case 171:
          if (!isViewerInPresentationMode) {
            _this8.pdfViewerApplication.zoomIn();
          }
          handled = true;
          break;
        case 173:
        case 109:
        case 189:
          if (!isViewerInPresentationMode) {
            _this8.pdfViewerApplication.zoomOut();
          }
          handled = true;
          break;
        case 48:
        case 96:
          if (!isViewerInPresentationMode) {
            setTimeout(function () {
              pdfViewer.currentScaleValue = _ui_utils.DEFAULT_SCALE_VALUE;
            });
            handled = false;
          }
          break;
        case 38:
          if (isViewerInPresentationMode || _this8.pdfViewerApplication.page > 1) {
            _this8.pdfViewerApplication.page = 1;
            handled = true;
            ensureViewerFocused = true;
          }
          break;
        case 40:
          if (isViewerInPresentationMode || _this8.pdfViewerApplication.page < _this8.pdfViewerApplication.pagesCount) {
            _this8.pdfViewerApplication.page = _this8.pdfViewerApplication.pagesCount;
            handled = true;
            ensureViewerFocused = true;
          }
          break;
      }
    }
    if (cmd === 1 || cmd === 8) {
      switch (evt.keyCode) {
        case 83:
          _this8.pdfViewerApplication.download();
          handled = true;
          break;
      }
    }
    if (cmd === 3 || cmd === 10) {
      switch (evt.keyCode) {
        case 80:
          _this8.pdfViewerApplication.requestPresentationMode();
          handled = true;
          break;
        case 71:
          _this8.pdfViewerApplication.appConfig.toolbar.pageNumber.select();
          handled = true;
          break;
      }
    }
    if (handled) {
      if (ensureViewerFocused && !isViewerInPresentationMode) {
        pdfViewer.focus();
      }
      evt.preventDefault();
      return;
    }
    var curElement = document.activeElement || document.querySelector(':focus');
    var curElementTagName = curElement && curElement.tagName.toUpperCase();
    if (curElementTagName === 'INPUT' || curElementTagName === 'TEXTAREA' || curElementTagName === 'SELECT') {
      if (evt.keyCode !== 27) {
        return;
      }
    }
    if (cmd === 0) {
      var turnPage = 0,
          turnOnlyIfPageFit = false;
      switch (evt.keyCode) {
        case 38:
        case 33:
          if (pdfViewer.isVerticalScrollbarEnabled) {
            turnOnlyIfPageFit = true;
          }
          turnPage = -1;
          break;
        case 8:
          if (!isViewerInPresentationMode) {
            turnOnlyIfPageFit = true;
          }
          turnPage = -1;
          break;
        case 37:
          if (pdfViewer.isHorizontalScrollbarEnabled) {
            turnOnlyIfPageFit = true;
          }
        case 75:
        case 80:
          turnPage = -1;
          break;
        case 27:
          if (_this8.pdfViewerApplication.secondaryToolbar) {
            if (_this8.pdfViewerApplication.secondaryToolbar.isOpen) {
              _this8.pdfViewerApplication.secondaryToolbar.close();
              handled = true;
            }
          }
          if (!_this8.pdfViewerApplication.supportsIntegratedFind && _this8.pdfViewerApplication.findBar.opened) {
            _this8.pdfViewerApplication.findBar.close();
            handled = true;
          }
          break;
        case 40:
        case 34:
          if (pdfViewer.isVerticalScrollbarEnabled) {
            turnOnlyIfPageFit = true;
          }
          turnPage = 1;
          break;
        case 13:
        case 32:
          if (!isViewerInPresentationMode) {
            turnOnlyIfPageFit = true;
          }
          turnPage = 1;
          break;
        case 39:
          if (pdfViewer.isHorizontalScrollbarEnabled) {
            turnOnlyIfPageFit = true;
          }
        case 74:
        case 78:
          turnPage = 1;
          break;
        case 36:
          if (isViewerInPresentationMode || _this8.pdfViewerApplication.page > 1) {
            _this8.pdfViewerApplication.page = 1;
            handled = true;
            ensureViewerFocused = true;
          }
          break;
        case 35:
          if (isViewerInPresentationMode || _this8.pdfViewerApplication.page < _this8.pdfViewerApplication.pagesCount) {
            _this8.pdfViewerApplication.page = _this8.pdfViewerApplication.pagesCount;
            handled = true;
            ensureViewerFocused = true;
          }
          break;
        case 83:
          _this8.pdfViewerApplication.pdfCursorTools.switchTool(_pdf_cursor_tools.CursorTool.SELECT);
          break;
        case 72:
          _this8.pdfViewerApplication.pdfCursorTools.switchTool(_pdf_cursor_tools.CursorTool.HAND);
          break;
        case 82:
          _this8.pdfViewerApplication.rotatePages(90);
          break;
      }
      if (turnPage !== 0 && (!turnOnlyIfPageFit || pdfViewer.currentScaleValue === 'page-fit')) {
        if (turnPage > 0) {
          if (_this8.pdfViewerApplication.page < _this8.pdfViewerApplication.pagesCount) {
            _this8.pdfViewerApplication.page++;
          }
        } else {
          if (_this8.pdfViewerApplication.page > 1) {
            _this8.pdfViewerApplication.page--;
          }
        }
        handled = true;
      }
    }
    if (cmd === 4) {
      switch (evt.keyCode) {
        case 13:
        case 32:
          if (!isViewerInPresentationMode && pdfViewer.currentScaleValue !== 'page-fit') {
            break;
          }
          if (_this8.pdfViewerApplication.page > 1) {
            _this8.pdfViewerApplication.page--;
          }
          handled = true;
          break;
        case 82:
          _this8.pdfViewerApplication.rotatePages(-90);
          break;
      }
    }
    if (!handled && !isViewerInPresentationMode) {
      if (evt.keyCode >= 33 && evt.keyCode <= 40 || evt.keyCode === 32 && curElementTagName !== 'BUTTON') {
        ensureViewerFocused = true;
      }
    }
    if (ensureViewerFocused && !pdfViewer.containsElement(curElement)) {
      pdfViewer.focus();
    }
    if (handled) {
      evt.preventDefault();
    }
  };
  this.apiPageModeToSidebarView = function (mode) {
    switch (mode) {
      case 'UseNone':
        return _pdf_sidebar.SidebarView.NONE;
      case 'UseThumbs':
        return _pdf_sidebar.SidebarView.THUMBS;
      case 'UseOutlines':
        return _pdf_sidebar.SidebarView.OUTLINE;
      case 'UseAttachments':
        return _pdf_sidebar.SidebarView.ATTACHMENTS;
      case 'UseOC':
    }
    return _pdf_sidebar.SidebarView.NONE;
  };
};

var PDFPrintServiceFactory = {
  instance: {
    supportsPrinting: false,
    createPrintService: function createPrintService() {
      throw new Error('Not implemented: createPrintService');
    }
  }
};
exports.PDFViewerApplication = PDFViewerApplication;
exports.DefaultExternalServices = DefaultExternalServices;
exports.PDFPrintServiceFactory = PDFPrintServiceFactory;
exports.PdfFileLoader = _pdf_file_loader.PdfFileLoader;